#property copyright "Scriptong"
#property link "http://autograf.dp.ua"

#property indicator_chart_window                   //     
#property indicator_buffers 4                      //    4- 
#property indicator_color1 Gray                    //    
#property indicator_color2 Blue                    //    
#property indicator_color3 Gray                    //    
#property indicator_color4 Red                     //    

#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 1
#property indicator_width4 1

extern ENUM_TIMEFRAMES TimeFrame=PERIOD_CURRENT;
extern int fractalPeriod = 5;

int center;

double upFractal[];                                //    
double upSignal[];                                 // ,   

double dnFractal[];                                //    
double dnSignal[];                                 // ,   

#define PREFIX "BRFRLI_"

//+-------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                            |
//+-------------------------------------------------------------------------------------+
int init()
{
   TuningParametersCheck();
   TimeFrame=MathMax(TimeFrame,_Period);

// - 1 - ==    ,   ==========================
   SetIndexBuffer(0, upFractal);                   //   -  
   SetIndexStyle(0, DRAW_ARROW);                   //     ..
   SetIndexArrow(0, 217);                          // .. 
   
   SetIndexBuffer(1, upSignal);                    //   -  
   SetIndexStyle(1, DRAW_ARROW);                   //     ..
   SetIndexArrow(1, 241);                          // .. 

   SetIndexBuffer(2, dnFractal);                   //   -  
   SetIndexStyle(2, DRAW_ARROW);                   //     ..
   SetIndexArrow(2, 218);                          // .. 

   SetIndexBuffer(3, dnSignal);                    //   -  
   SetIndexStyle(3, DRAW_ARROW);                   //     ..
   SetIndexArrow(3, 242);                          // .. 
// - 1 - ==   =============================================================

   return(0);
}
//+-------------------------------------------------------------------------------------+
//|                                      |
//+-------------------------------------------------------------------------------------+
void TuningParametersCheck()
{
   string name = WindowExpertName();
   if (fractalPeriod < 3)
   {
      Alert(name, ":       3!   3.");
      fractalPeriod = 3;
   }
   if (fractalPeriod%2 == 0)
   {
      fractalPeriod++;
      Alert(name, ":     !   ", 
            fractalPeriod, ".");
   }
   center = fractalPeriod/2 + 1;
}
//+-------------------------------------------------------------------------------------+
//| Custom indicator deinitialization function                                          |
//+-------------------------------------------------------------------------------------+
int deinit()
{
   for (int i = ObjectsTotal()-1; i >= 0; i--)     //    
      if (StringSubstr(ObjectName(i), 0, StringLen(PREFIX)) == PREFIX)
         ObjectDelete(ObjectName(i));
   return(0);
}
//+-------------------------------------------------------------------------------------+
//|   ,                   |
//+-------------------------------------------------------------------------------------+
int GetRecalcIndex()
{
   int counted_bars = IndicatorCounted();
   if (counted_bars == 0)                          // -   - 0. 
   {                                               // ..    ..
      ArrayInitialize(upFractal, EMPTY_VALUE);     // ...  
      ArrayInitialize(upSignal, EMPTY_VALUE);
      ArrayInitialize(dnFractal, EMPTY_VALUE);
      ArrayInitialize(dnSignal, EMPTY_VALUE);
      return(Bars - center);                       //     ,..
                                                   // ..   
                                                   // ..   
   }
   return(Bars - counted_bars - 1);                //    
}
//+-------------------------------------------------------------------------------------+
//|                                       |
//+-------------------------------------------------------------------------------------+
bool IsUpFractal(int index)
{
// - 1 - ==         =================
   for (int i = 1; i < center; i++)                //      index 
      if (High[index] <= High[index-i])            // ..   ..
         return(false);                            // ..  index
// - 1 - ==   =============================================================

// - 2 - ==         ==================
   int equalBars = 0;                              // -    High
   for(i = 1; i < center+equalBars && index+i < Bars; i++)//     index
   {                                               // ..   ..
      if (High[index] == High[index+i])            // ..  index,   ..
         equalBars++;                              // ..   
      if (High[index] < High[index+i])             
         return(false);                            
   }
// - 2 - ==   =============================================================

   return(true);                                   // index   
}
//+-------------------------------------------------------------------------------------+
//|                                        |
//+-------------------------------------------------------------------------------------+
bool IsDnFractal(int index)
{
// - 1 - ==         =================
   for (int i = 1; i < center; i++)                //      index 
      if (Low[index] >= Low[index-i])              // ..   ..
         return(false);                            // ..  index
// - 1 - ==   =============================================================

// - 2 - ==         ==================
   int equalBars = 0;                              // -    Low
   for(i = 1; i < center+equalBars && index+i < Bars; i++)//     index
   {                                               // ..   ..
      if (Low[index] == Low[index+i])              // ..  index,   ..
         equalBars++;                              // ..   
      if (Low[index] > Low[index+i])
         return(false);
   }
// - 2 - ==   =============================================================

   return(true);                                   //  index   
}
//+-------------------------------------------------------------------------------------+
//|                                                    |
//+-------------------------------------------------------------------------------------+
void FractalsFind(int index)
{
   for (int i = index+center; i >= center; i--)    //   upFractal ..
   {                                               // ..dnFractal
      if (IsUpFractal(i))                          //    ..
         upFractal[i] = High[i];                   //..  
      if (IsDnFractal(i))                          //    ..
         dnFractal[i] = Low[i];                    // ..   
   }
}
//+-------------------------------------------------------------------------------------+
//|                                           |
//+-------------------------------------------------------------------------------------+
void GetFractal(int index, double fractal, int &barIndex, double &value, double &kKoef)
{
   if (fractal == EMPTY_VALUE)                     //   -   
      return;
      
   barIndex = index;                               //   -  
   value = fractal;                                //   
   kKoef = 0;                                      // ..  ,    ..
                                                   // ..  
}
//+-------------------------------------------------------------------------------------+
//|   K  B                               |
//+-------------------------------------------------------------------------------------+
double GetKAndBFromCoords(int x1, double y1, int x2, double y2, double &bKoef)
{
   if (x1 - x2 == 0 || x1 == 0)                    //     0
   {
      bKoef = 0;                                   //    
      return (0);
   }
   
   bKoef = (y2*x1 - y1*x2)/(x1 - x2);              //  B  
   return ((y1 - bKoef)/x1);                       //    
}
//+-------------------------------------------------------------------------------------+
//|                        |
//+-------------------------------------------------------------------------------------+
void LastFractals(int index, int &numUp, int &numDn, double &valUp, double &valDn,
                  double &kKoefUp, double &kKoefDn, double &bKoefUp, double &bKoefDn,
                  int &lastUp, int &lastDn)
{
   for (int i = index+center;                      //      ,
        i < Bars && (numUp == 0 || numDn == 0);    // ..    ..
        i++)                                       // ..    
   {
   // - 1 - ==    ==================================================
      if (numUp == 0)                              //      
      {
         GetFractal(i, upFractal[i], numUp, valUp, kKoefUp);//   
         if (numUp > 0)                            //  ?
         {
            for (int k = i-center; k > index; k--) //     ?
               if (High[k] > valUp)                //  
               {
                  kKoefUp = GetKAndBFromCoords(k,  //   
                                               High[k], numUp, valUp, bKoefUp);
                  for (int j = k-1; j > index && lastUp == 0; j--)//    ?
                     if (Close[j] > j*kKoefUp + bKoefUp)//  
                        lastUp = numUp;            //    
                  break;
               }
         }
      }
   // - 1 - ==   ==========================================================

   // - 2 - ==    ===================================================
      if (numDn == 0)                              //      
      {
         GetFractal(i, dnFractal[i], numDn, valDn, kKoefDn);//   
         if (numDn > 0)                            //  ?
         {
            for (k = i-center; k > index; k--)     //     ?
               if (Low[k] < valDn)                 //  
               {
                  kKoefDn = GetKAndBFromCoords(k,  //   
                                               Low[k], numDn, valDn, bKoefDn);
                  for (j = k-1; j > index && lastDn == 0; j--)//    ?
                     if (Close[j] < j*kKoefDn + bKoefDn)//  
                        lastDn = numDn;            //    
                  break;
               }
         }
      }
   // - 2 - ==   ==========================================================
   }
}
//+-------------------------------------------------------------------------------------+
//|                                                      |
//+-------------------------------------------------------------------------------------+
void ShowLine(datetime time1, double price1, datetime time2, double price2, 
              string direction)
{
// - 1 - ==       =======================================
   string name = PREFIX + time1 + direction;       //   
   if (direction == "UP")                          //     - 
      color clr = indicator_color2;                // ..,.. 
   else                                            // ..     - ..
      clr = indicator_color4;                      // .. 
// - 1 - ==   =============================================================

// - 2 - ==   ============================================================
   if (ObjectFind(name) < 0)                       //      , 
   {
      ObjectCreate(name, OBJ_TREND, 0, time1, price1, time2, price2);// .. 
      ObjectSet(name, OBJPROP_COLOR, clr);         //   
      ObjectSet(name, OBJPROP_RAY, false);         //    
      ObjectSet(name, OBJPROP_STYLE, STYLE_DOT);   //   -  
      return;  
   }
// - 2 - ==   =============================================================
   
// - 3 - ==   =========================================================
   ObjectMove(name, 0, time1, price1);    
   ObjectMove(name, 1, time2, price2);
// - 3 - ==   =============================================================
}
//+-------------------------------------------------------------------------------------+
//|  ,                         |
//+-------------------------------------------------------------------------------------+
void ProcessIt(int index)
{
// - 1 - ==       ======================
   int numUp = 0, numDn = 0;                       //    
   double valUp = 0, valDn = 0;                    //  
   double kKoefUp = 0, kKoefDn = 0;                //    ,..
                                                   // ..   
   double bKoefUp = 0, bKoefDn = 0;                //    
   int lastUp = 0, lastDn = 0;                     //    
   LastFractals(index, numUp, numDn, valUp, valDn, //   
                kKoefUp, kKoefDn, bKoefUp, bKoefDn,
                lastUp, lastDn);
// - 1 - ==   =============================================================

// - 2 - ==    ======================================================
   for (int i = index; i > 0; i--)                 //   
   {
   // - 2.1 - ==     =================================
      GetFractal(i+center, upFractal[i+center], numUp, valUp, kKoefUp);
      GetFractal(i+center, dnFractal[i+center], numDn, valDn, kKoefDn);
   // - 2.1 - ==   ========================================================
      
   // - 2.2 - ==     ===================================
      if (kKoefUp == 0)                            //    
         if (High[i] > valUp)                      //    ?
            kKoefUp = GetKAndBFromCoords(i,        //   ,
                                         High[i],  // ..   ..
                                         numUp,    // ..  
                                         valUp, bKoefUp);
      if (kKoefDn == 0)                            //    
         if (Low[i] < valDn)                       //    ?
            kKoefDn = GetKAndBFromCoords(i, Low[i],//   ,..
                                         numDn,    // ..   ..
                                         valDn,    // ..  
                                         bKoefDn);
   // - 2.2 - ==   ========================================================
         
   // - 2.3 - ==     ========================================
      if (kKoefUp != 0 && lastUp != numUp)         //    
         if (Close[i] > i*kKoefUp + bKoefUp)       //     
         {
            upSignal[i-1] = Low[i-1] - iATR(NULL, 0, 14, i-1)/2;//  
            ShowLine(Time[numUp], valUp, Time[i],  //    ..
                     i*kKoefUp + bKoefUp, "UP");   // ..
            lastUp = numUp;                        // ,    
         }
      if (kKoefDn != 0 && lastDn != numDn)         //    
         if (Close[i] < i*kKoefDn + bKoefDn)       //     
         {  
            dnSignal[i-1] = High[i-1] + iATR(NULL, 0, 14, i-1)/2;//  
            ShowLine(Time[numDn], valDn, Time[i],  //    ..
                     i*kKoefDn + bKoefDn, "DN");   // ..
            lastDn = numDn;                        // ,   
         }
   // - 2.3 - ==   ========================================================
   }
// - 2 - ==   =============================================================
}
//+-------------------------------------------------------------------------------------+
//| Custom indicator iteration function                                                 |
//+-------------------------------------------------------------------------------------+
int start()
{
      if (TimeFrame!=_Period)
      {
         string indicatorName=WindowExpertName();
         for (int i =Bars-1; i > 0; i--)
         {
            int y=iBarShift(NULL,TimeFrame,Time[i]);
            int x=iBarShift(NULL,TimeFrame,Time[MathMin(i+1,Bars-1)]);
            if (x!=y)
            {
               upFractal[i] = iCustom(NULL,TimeFrame,indicatorName,0,fractalPeriod,0,y);
               upSignal[i]  = iCustom(NULL,TimeFrame,indicatorName,0,fractalPeriod,1,y);
               dnFractal[i] = iCustom(NULL,TimeFrame,indicatorName,0,fractalPeriod,2,y);
               dnSignal[i]  = iCustom(NULL,TimeFrame,indicatorName,0,fractalPeriod,3,y);
            }
            else { upFractal[i] = EMPTY_VALUE; upSignal[i] = EMPTY_VALUE; dnFractal[i] = EMPTY_VALUE; dnSignal[i] = EMPTY_VALUE; }
         }
         return(0);
      }
   int limit = GetRecalcIndex();                   //    
   FractalsFind(limit);                            //    
   ProcessIt(limit);                               //    
   return(0);
}